#include <fstream>
#include <iostream>
#include <sstream>
#include <vector>


#include <opencv2/opencv.hpp>

using namespace cv;
using namespace std;

int main(int argc, char** argv)
{
    Size patternSize(6, 9);
    Size imageSize;
    
    vector<vector<Point2f> > imagePointLocs;
    
    for (int i = 1; i < argc; ++i)
    {
        Mat calibImage = imread(argv[i]);
        imageSize = calibImage.size();
        vector<Point2f> corners;
        stringstream s;
        s << i;
        
        if (calibImage.empty()) {
            cerr << "Error: Failed to load image " << argv[i] << endl;
            continue;
        }

        if (!findChessboardCorners(calibImage, patternSize, corners)) {
            cerr << "Failed to detect corners in image "
                 << argv[i] << endl;
            continue;
        }
        
        for (size_t c = 0; c < 4; ++c)
            circle(calibImage, corners[c], 5, Scalar(255, 0, 0), -1);

        imagePointLocs.push_back(corners);
        
//        imshow(s.str(), calibImage);
    }
    
    vector<Point3f> pointLocs;
    for (int x = 8; x >= 0; --x)
        for (int y = 5; y >= 0; --y)
            pointLocs.push_back(Point3f(x, y, 0));


    vector<vector<Point3f> > multiPointLocs;
    while (multiPointLocs.size() < imagePointLocs.size())
        multiPointLocs.push_back(pointLocs);

    Mat cameraMatrix, distCoeffs;
    vector<Mat> rvecs, tvecs;
    
    cout << "Calibrating camera... " << flush;
    calibrateCamera(multiPointLocs, imagePointLocs, imageSize,
                    cameraMatrix, distCoeffs, rvecs, tvecs);
    cout << "Done.\n" << endl;
    
    for (int i = 1; i < argc; ++i)
    {
        stringstream s;
        s << i;
        
        Mat testImg = imread(argv[i]);
        Mat outImg;
//        imshow("Distorted", testImg);
        undistort(testImg, outImg, cameraMatrix, distCoeffs);
//        imshow("Undistorted", outImg);
        imwrite("undistorted/" + s.str() + ".jpg", outImg);
    }
    
    
    cout << "Camera matrix:\n" << cameraMatrix << endl << endl
         << "Distortion coefficients\n" << distCoeffs << endl << endl;
    
    ofstream file("calibration.txt");

    for (int i = 0; i < 3; ++i)
        for (int j = 0; j < 3; ++j)
            file << "\tcameraMatrix.at<double>(" << i << ", " << j << ") = "
                 << cameraMatrix.at<double>(i, j) << ";\n";
    file << "\n";
    for (int i = 0; i < 5; ++i)
        file << "\tdistCoeffs.at<double>(0, " << i << ") = "
             << distCoeffs.at<double>(0, i) << ";\n";

    file << "\tfocalLength = " << 
        (cameraMatrix.at<double>(0, 0) + cameraMatrix.at<double>(1, 1)) / 2
        << ";\n";
    file.close();
    
    cout << "Matrices written to 'calibration.txt'." << endl;
//    while(waitKey(10) < 0);
}
        
    