#include <fstream>
#include <iostream>
#include <sstream>
#include <vector>

#include "cylinder.hpp"
#include "mask.hpp"

using namespace cv;
using namespace std;

extern cv::Mat cameraMatrix;
extern cv::Mat distCoeffs;

Mat cylinderize(Mat in, double focalLength)
{   
    double minTheta = atan(-in.cols / (2*focalLength)),
           maxTheta = atan( in.cols / (2*focalLength));
           
    double minH = -in.rows / 2,
           maxH =  in.rows / 2;
    
    Mat out(in.size(), in.type(), Scalar(0, 0, 0));

    // Loop through all of the pixels in the target image.
    for (int i = 0; i < out.cols; ++i)
        for (int j = 0; j < out.rows; ++j) {

            // Figure out cylindrical projection values.
            double theta = i * (maxTheta - minTheta) / out.cols  + minTheta;
            double h = j * (maxH - minH) / out.rows + minH;
            
            // Then work backwards to the original picture.
            double xOrig = tan(theta) * focalLength + in.cols / 2;
            double yOrig = h / cos(theta) + in.rows / 2;
           
            // If this reprojected point is within the original image, then
            // copy it over.            
            if (xOrig >= 0 && xOrig < in.cols &&
                yOrig >= 0 && yOrig < in.rows)
            {     
                if (in.type() == CV_8UC3)
                    out.at<Vec3b>(j, i) = in.at<Vec3b>(int(yOrig), int(xOrig));
                else if (in.type() == CV_8UC4)
                    out.at<Vec4b>(j, i) = in.at<Vec4b>(int(yOrig), int(xOrig));
                else if (in.type() == CV_16UC3)
                    out.at<Vec3s>(j, i) = in.at<Vec3s>(int(yOrig), int(xOrig));
                else if (in.type() == CV_16UC4)
                    out.at<Vec4s>(j, i) = in.at<Vec4s>(int(yOrig), int(xOrig));
                else
                    cout << "Unknown type being passed into cylinderize."
                         << endl;
                    
            }

        }
    return tightenMask(out);
}

// This double comes from the calibration.cpp file.
extern double focalLength;

Mat cylinderize(Mat in)
{
    return cylinderize(in, focalLength);
}

// Convert a single point into cylindrical coordinates.
Point2f ToCylinder(double xOrig, double yOrig, Size sizeIn, Size sizeOut)
{
    // Values used for the cylindrical projection process.
    double minTheta = atan(-sizeOut.width / (2*focalLength)),
           maxTheta = atan( sizeOut.width / (2*focalLength));
           
    double minH = -sizeOut.height / 2,
           maxH =  sizeOut.height / 2;
    
    double theta = atan( (xOrig - sizeIn.width / 2.) /
                    focalLength);
    double h = (yOrig - sizeIn.height / 2) * cos(theta);
    
    Point2i out;
    
    out.x = int((theta - minTheta) * sizeOut.width / (maxTheta - minTheta));
    out.y = int((h - minH) * sizeOut.height / (maxH - minH));

    return out;
}

Point2f ToCylinder(Point2f orig, Size sizeIn, Size sizeOut)
{
    return ToCylinder(orig.x, orig.y, sizeIn, sizeOut);
}