#include <list>
#include <stdlib.h>
#include <sstream>

#include "corner.hpp"
#include "feature.hpp"
#include "mask.hpp"

using namespace cv;
using namespace std;

Feature::Feature(Mat& feature, Point2f l)
    : f(feature), loc(l)
{
    // Nothing to do here.
}

// Extract features from an image, given a set of corners.
vector<Feature> getFeatures(Mat& im, vector<Point2f> corners)
{
    vector<Feature> features;

    Mat im8u, bwImage;
    // If necessary, convert from masked image into uchars.
    if (im.type() == CV_16UC4) {
        im8u = maskedTo8U(im);
    }
    else
        im8u = im;
    
    
    cvtColor(im8u, bwImage, CV_BGR2GRAY);   
    
    Mat blurred;
    GaussianBlur(bwImage, blurred, Size(), 3);
    
    vector<Point2f>::iterator c;
    
    Rect boundingRect = Rect(20, 20,
                             blurred.size().width - 40, blurred.size().height - 40);
                             
    for (c = corners.begin(); c != corners.end(); ++c) {

        if (!boundingRect.contains(*c))
            continue;
        
        Mat feature;
        // Pull out a 40x40 blurred feature, then resize it to 8x8 and save it.
        resize(Mat(blurred, Rect((*c) - Point2f(20, 20),
                                 (*c) + Point2f(20, 20))),
               feature, Size(8, 8));

        Scalar mean(0), stddev(0);
        meanStdDev(feature, mean, stddev);
        
        // Make a normalized version of the feature, to aid in matching.
        Mat featureNorm;
        feature.convertTo(featureNorm, CV_64F,
                          1.0/stddev[0], -mean[0] / stddev[0]);

        features.push_back(Feature(featureNorm, *c));
    }
    
    return features;
}

vector<Feature> getFeatures(Mat& im)
{
    vector<Point2f> corners = findCorners(im);
    return getFeatures(im, corners);
}