#include "helpers.hpp"
#include "mask.hpp"

using namespace cv;
using namespace std;

// Rescales an image so that its maximum dimension is maxSize, preserving aspect
// ratio.
void rescale(Mat& A, Mat& B, int maxSize)
{
    
    if (A.size().width > maxSize || A.size().height > maxSize)
    {
        double scaleFactor = double(maxSize) / max(A.size().width,
                                                   A.size().height);
        
        resize(A, B, Size(scaleFactor * A.size().width,
                          scaleFactor * A.size().height));
    } else
        A.copyTo(B);
}

Mat expand(Mat& im, Size newSize, Point2f offset)
{
    Mat resized(newSize, im.type(), Scalar(0));
    Rect roi(offset.x, offset.y, im.size().width, im.size().height);
    Mat target(resized, roi);
    im.copyTo(target);
    return resized;
}

Mat tightCrop(Mat& inColor)
{
    double maxVal;
    
    Mat in;

    cvtColor(inColor, in, CV_BGR2GRAY);  

    int minRow = 0, maxRow = in.size().height,
        minCol = 0, maxCol = in.size().width;
    
    // Find the first non-zero row
    for(int row = 0; row < in.size().height; ++row) {
        minMaxLoc(in.row(row), NULL, &maxVal);
        if (maxVal == 0)
            minRow = row;
        else
            break;
    }
    // Find the earliest non-zero row from the back
    for(int row = in.size().height - 1; row >= 0; --row) {
        minMaxLoc(in.row(row), NULL, &maxVal);
        if (maxVal == 0)
            maxRow = row;
        else
            break;
    }

    // Find the first non-zero col
    for(int col = 0; col < in.size().width; ++col) {
        minMaxLoc(in.col(col), NULL, &maxVal);
        if (maxVal == 0)
            minCol = col;
        else
            break;
    }
    
    // Find the earliest non-zero col from the back
    for(int col = in.size().width - 1; col >= 0; --col) {
        minMaxLoc(in.col(col), NULL, &maxVal);
        if (maxVal == 0)
            maxCol = col;
        else
            break;
    }

    return Mat(inColor, Rect(minCol, minRow,
                             maxCol - minCol, maxRow - minRow));
}