#include <vector>

#include "mask.hpp"

using namespace std;
using namespace cv;

Mat extractMask(Mat in)
{
    if (in.depth() != 4)
    {
        cerr << "Warning: No mask to remove";
        return Mat(in.size(), CV_8U, Scalar(255));
    }

    // Split the matrix up.    
    vector<Mat> mv;
    split(in, mv);

    return mv[3];
}

Mat removeMask(Mat in)
{
    if (in.depth() != 4)
    {
        cerr << "Warning: No mask to remove";
        return in;
    }

    // Split the matrix up.    
    vector<Mat> mv;
    split(in, mv);

    // Remove the mask channel and recombine.
    mv.pop_back();
    Mat out;
    merge(mv, out);
    
    return out;   
}

Mat addMask(Mat in, Mat inMask)
{
    if (in.depth() == 4)
    {
        cerr << "Warning: Mask is already on!";
        return in;
    }

    // Split the matrix up.    
    vector<Mat> mv;
    split(in, mv);
    Mat mask;
    inMask.copyTo(mask);

    // Add the mask channel and recombine.
    mv.push_back(mask);
    
    // Scale everything into 16-bit datatype.
    for(size_t m = 0; m < mv.size(); ++m)
        if (mv[m].type() == CV_8U)
            mv[m].convertTo(mv[m], CV_16U);
            
    Mat out;
    merge(mv, out);
    
    return out;           
}

Mat addMask(Mat in)
{
    return addMask(in, Mat(in.size(), CV_16U, 1));
}

Mat tightenMask(Mat& in)
{
    vector<Mat> mv;
    split(in, mv);
    
    mv[3].convertTo(mv[3], CV_8U);
    threshold(mv[3], mv[3], 0, 1, THRESH_BINARY);
    erode(mv[3], mv[3], Mat());
    mv[3].convertTo(mv[3], CV_16U);
    
    Mat combined;
    merge(mv, combined);
    return combined;
}

Mat maskedTo8U(Mat& in)
{
    Mat inCopy, out;
    in.copyTo(inCopy);
    vector<Mat> mv;
    
    split(inCopy, mv);

    for(int i = 0; i < 3; ++i)
        mv[i] = mv[i].mul(1/mv[3]);
    
//    mv[3] = mv[3].mul(255/mv[3]);
    mv[3].convertTo(mv[3], CV_8U);
    threshold(mv[3], mv[3], 0, 255, THRESH_BINARY);
    mv[3].convertTo(mv[3], CV_16U);
    
    merge(mv, out);
    
    out.convertTo(out, CV_8UC4);
    return out;
}

Mat combineMasked(Mat im1, Mat im2)
{
    vector<Mat> mv1;
    split(im1, mv1);
    vector<Mat> mv2;
    split(im2, mv2);
    
    vector<Mat> combinedV;
    
    Mat mask1, mask2;
    mv1[3].copyTo(mask1);
    mv2[3].copyTo(mask2);
    
    mask1.convertTo(mask1, CV_8U);
    mask2.convertTo(mask2, CV_8U);
    threshold(mask1, mask1, 0, 1, THRESH_BINARY);
    threshold(mask2, mask2, 0, 1, THRESH_BINARY);
    mask1.convertTo(mask1, CV_16U, (2 << 16) - 1);
    mask2.convertTo(mask2, CV_16U, (2 << 16) - 1);
    
    for(int i = 0; i < 3; ++i)
        combinedV.push_back( (mv1[i] & mask1) + (mv2[i] & mask2));

    // Find new mask.
    combinedV.push_back(mv1[3] + mv2[3]);

    Mat combined;
    merge(combinedV, combined);
    return combined;
}