#include "match.hpp"
#include "feature.hpp"

using namespace std;
using namespace cv;

Match::Match(Point2f l1)
    : loc1(l1), loc2(Point2f(-1, -1)), bestMatch(DBL_MAX), secondMatch(DBL_MAX)
{
    // Nothing to do here.
}

bool Match::operator<(const Match& m) const
{
    return (bestMatch / secondMatch) < (m.bestMatch / m.secondMatch);
}

vector<Match> getMatches(vector<Feature>& f1, vector<Feature>& f2)
{
    vector<Match> matches;
    
    Mat ssd(f1[0].f.size(), CV_64F, Scalar(0));
    Scalar m;

    // Go through every pair of features and check to see how well they line up.
    for (size_t i = 0; i < f1.size(); ++i) {
        Match currentMatch(f1[i].loc);
        
        for (size_t j = 0; j < f2.size(); ++j) {
            
            // Save the squared distance.
            ssd = 0.;
            accumulateSquare(f1[i].f - f2[j].f, ssd);
            Scalar m  = mean(ssd);
            
            if (m[0] < currentMatch.bestMatch) {
                currentMatch.secondMatch = currentMatch.bestMatch;
                currentMatch.bestMatch = m[0];
                currentMatch.loc2 = f2[j].loc;
            }
        }
        matches.push_back(currentMatch);         
    }
    
    // Matches are sorted based on the ratio of best to second-best match.
    // Lower == better.
    sort(matches.begin(), matches.end());

    while ((matches.back().bestMatch / matches.back().secondMatch) > 0.5)
        matches.pop_back();

    return matches;
}

void drawMatches(Mat& im1, Mat& im2, vector<Match> matches, unsigned int n)
{
    Mat im1b, im2b;
    im1.copyTo(im1b);
    im2.copyTo(im2b);
    
    for (size_t i = 0; i < matches.size() && i < n; ++i)
    {
        Scalar color;
        switch (i % 5)
        {
            case 0:
                color = Scalar(0, 0, 255);
            break;
            case 1:
                color = Scalar(0, 255, 255);
            break;
            case 2:
                color = Scalar(0, 255, 0);
            break;
            case 3:
                color = Scalar(255, 255, 0);
            break;
            case 4:
                color = Scalar(255, 0, 0);
            break;
            default:
                color = Scalar(255, 255, 255);
        }
        if (n > 5)
            color = Scalar(0, 0, 255);
                                   
        circle(im1b, matches[i].loc1, 4, color, -1);
        circle(im2b, matches[i].loc2, 4, color, -1);
    }
    imshow("matches1", im1b);
    imshow("matches2", im2b);
}